==> egg-user-service-1.0.0.7-sources.jar: com/bxm/egg/user/auth/impl/UserAuthCodeServiceImpl.java <==
package com.bxm.egg.user.auth.impl;

import com.bxm.egg.common.enums.AuthCodeEnum;
import com.bxm.egg.common.enums.RoleCodeEnum;
import com.bxm.egg.user.auth.UserAuthCodeService;
import com.bxm.egg.user.constant.AuthRedisKey;
import com.bxm.egg.user.facade.dto.UserAuthResourceDTO;
import com.bxm.egg.user.integration.MessageFacadeIntegrationService;
import com.bxm.egg.user.mapper.auth.UserAuthCodeMapper;
import com.bxm.egg.user.mapper.auth.UserAuthIdMapper;
import com.bxm.egg.user.mapper.auth.UserRoleAuthMapper;
import com.bxm.egg.user.mapper.auth.UserRoleMapper;
import com.bxm.egg.user.model.vo.auth.UserAuthCodeBean;
import com.bxm.egg.user.model.vo.auth.UserAuthIdBean;
import com.bxm.egg.user.model.vo.auth.UserRoleAuthBean;
import com.bxm.egg.user.model.vo.auth.UserRoleBean;
import com.bxm.newidea.component.bo.Message;
import com.bxm.newidea.component.redis.KeyGenerator;
import com.bxm.newidea.component.redis.RedisHashMapAdapter;
import com.bxm.newidea.component.tools.BitOperatorUtil;
import com.bxm.newidea.component.tools.StringUtils;
import com.bxm.newidea.component.uuid.config.SequenceHolder;
import com.fasterxml.jackson.core.type.TypeReference;
import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
public class UserAuthCodeServiceImpl implements UserAuthCodeService {

    private RedisHashMapAdapter redisHashMapAdapter;

    private UserAuthCodeMapper userAuthCodeMapper;

    private UserAuthIdMapper userAuthIdMapper;

    private UserRoleAuthMapper userRoleAuthMapper;

    private UserRoleMapper userRoleMapper;

    private MessageFacadeIntegrationService messageSender;

    public UserAuthCodeServiceImpl(RedisHashMapAdapter redisHashMapAdapter,
                                   UserAuthCodeMapper userAuthCodeMapper,
                                   UserAuthIdMapper userAuthIdMapper,
                                   UserRoleAuthMapper userRoleAuthMapper,
                                   UserRoleMapper userRoleMapper,
                                   MessageFacadeIntegrationService messageSender) {
        this.redisHashMapAdapter = redisHashMapAdapter;
        this.userAuthCodeMapper = userAuthCodeMapper;
        this.userAuthIdMapper = userAuthIdMapper;
        this.userRoleAuthMapper = userRoleAuthMapper;
        this.userRoleMapper = userRoleMapper;
        this.messageSender = messageSender;
    }

    private TypeReference<List<String>> typeReference = new TypeReference<List<String>>() {
    };

    @Override
    public Long[] getAuthCombineCode(Long userId) {
        String combineCode = redisHashMapAdapter.get(AuthRedisKey.USER_AUTH_CODE, userId.toString(), String.class);

        Long[] authCodeArray;

        if (null == combineCode) {
            authCodeArray = new Long[]{0L};
        } else {
            String[] authCodeStrArray = combineCode.split(",");
            authCodeArray = new Long[authCodeStrArray.length];

            for (int i = 0; i < authCodeStrArray.length; i++) {
                authCodeArray[i] = Long.valueOf(authCodeStrArray[i]);
            }
        }

        return authCodeArray;
    }

    @Override
    public List<UserAuthResourceDTO> getAuthResources(Long userId) {
        Map<String, List<String>> authResourceMap = redisHashMapAdapter.entries(buildKey(userId), typeReference);

        if (null == authResourceMap) {
            authResourceMap = reloadResourceIdCache(userId);
        }

        return authResourceMap.entrySet().stream().map(entry ->
                new UserAuthResourceDTO(Integer.valueOf(entry.getKey()), entry.getValue()))
                .collect(Collectors.toList());
    }

    /**
     * 重新加载权限ID缓存数据
     *
     * @param userId 用户ID
     * @return 缓存数据
     */
    private Map<String, List<String>> reloadResourceIdCache(Long userId) {
        Map<String, List<String>> cacheMap = Maps.newHashMap();

        List<UserAuthIdBean> userAuthIdBeans = userAuthIdMapper.queryAuthIds(userId);

        userAuthIdBeans.forEach(item -> {
            String key = item.getAuthCode() + "";
            List<String> resourceIds = cacheMap.get(key);
            if (null == resourceIds) {
                resourceIds = Lists.newArrayList();
            }
            resourceIds.add(item.getAuthId());
            cacheMap.put(key, resourceIds);
        });

        KeyGenerator key = buildKey(userId);
        redisHashMapAdapter.remove(key);
        redisHashMapAdapter.putAll(key, cacheMap);

        return cacheMap;
    }

    @Override
    public Message addRole(Long userId, RoleCodeEnum role) {
        Preconditions.checkArgument(null != role);

        //设置用户拥有的角色
        UserRoleBean roleBean = new UserRoleBean();
        roleBean.setId(SequenceHolder.nextLongId());
        roleBean.setCreateTime(new Date());
        roleBean.setRoleCode(role.name());
        roleBean.setUserId(userId);

        userRoleMapper.insert(roleBean);

        //获取角色对应的操作权限列表
        List<UserRoleAuthBean> authBeans = userRoleAuthMapper.queryByCode(role.name());
        //设置用户拥有的权限集合
        Long[] authCombineCode = getAuthCombineCode(userId);
        for (UserRoleAuthBean userRoleAuth : authBeans) {
            AuthCodeEnum codeEnum = AuthCodeEnum.getByIndex(userRoleAuth.getAuthCode());
            authCombineCode = changeUserAuthCode(authCombineCode, codeEnum, true);
        }

        Message message = saveAuthCode(authCombineCode, userId);

        return message;
    }

    @Override
    public Message removeRole(Long userId, RoleCodeEnum role) {
        Preconditions.checkArgument(null != role);
        Preconditions.checkArgument(null != userId);

        //删除用户拥有的角色
        userRoleMapper.removeUserRole(userId, role.name());

        //获取角色对应的操作权限列表
        List<UserRoleAuthBean> authBeans = userRoleAuthMapper.queryByCode(role.name());
        //设置用户拥有的权限集合
        Long[] authCombineCode = getAuthCombineCode(userId);
        for (UserRoleAuthBean userRoleAuth : authBeans) {
            AuthCodeEnum codeEnum = AuthCodeEnum.getByIndex(userRoleAuth.getAuthCode());
            authCombineCode = changeUserAuthCode(authCombineCode, codeEnum, false);
        }

        //移除用户拥有的权限-资源信息
        //批量操作
        for (UserRoleAuthBean authBean : authBeans) {
            userAuthIdMapper.removeAuth(userId, authBean.getAuthCode());
        }

        reloadResourceIdCache(userId);

        return saveAuthCode(authCombineCode, userId);
    }

    @Override
    public Message addAuthCode(Long userId, AuthCodeEnum authCode) {
        Long[] authCombineCode = getAuthCombineCode(userId);
        authCombineCode = changeUserAuthCode(authCombineCode, authCode, true);

        return saveAuthCode(authCombineCode, userId);
    }

    private Message saveAuthCode(Long[] authCodeArray, Long userId) {
        String authCombineCode = StringUtils.join(authCodeArray, ",");
        redisHashMapAdapter.put(AuthRedisKey.USER_AUTH_CODE, userId.toString(), authCombineCode);

        int rows;
        UserAuthCodeBean entity = userAuthCodeMapper.queryByUserId(userId);
        if (entity == null) {
            entity = new UserAuthCodeBean();
            entity.setId(SequenceHolder.nextLongId());
            entity.setCreateTime(new Date());
            entity.setAuthCombineCode(authCombineCode);
            entity.setUserId(userId);

            rows = userAuthCodeMapper.insert(entity);
        } else {
            entity.setAuthCombineCode(authCombineCode);
            entity.setModifyTime(new Date());

            rows = userAuthCodeMapper.updateByPrimaryKey(entity);
        }

        return Message.build(rows);
    }

    /**
     * 变更用户持有的权限合集
     *
     * @param authCombineCode 权限编码合集
     * @param authCode        变更的授权编码
     * @param isAdd           true表示添加，false表示移除
     * @return 变更后的权限合集
     */
    private Long[] changeUserAuthCode(Long[] authCombineCode, AuthCodeEnum authCode, boolean isAdd) {
        return BitOperatorUtil.setBitToArray(authCombineCode, authCode.index, isAdd);
    }

    @Override
    public Message removeAuthCode(Long userId, AuthCodeEnum authCode) {
        Long[] authCombineCode = getAuthCombineCode(userId);
        authCombineCode = changeUserAuthCode(authCombineCode, authCode, false);

        userAuthIdMapper.removeAuth(userId, authCode.index);
        redisHashMapAdapter.remove(buildKey(userId), authCode.index + "");

        return saveAuthCode(authCombineCode, userId);
    }

    @Override
    public Message addResource(Long userId, AuthCodeEnum authCode, String resourceId) {
        return addResource(userId, authCode, ImmutableList.of(resourceId));
    }

    @Override
    public Boolean hasAuth(Long userId, AuthCodeEnum codeEnum) {
        int authCode = codeEnum.index;
        Long[] parseAuthCodeArray = getAuthCombineCode(userId);

        return BitOperatorUtil.hasBit(parseAuthCodeArray, authCode);
    }

    @Override
    public Message removeResource(Long userId, AuthCodeEnum authCode, String resourceId) {
        KeyGenerator key = buildKey(userId);
        String subKey = String.valueOf(authCode.index);

        List<String> resources = redisHashMapAdapter.get(key, subKey, typeReference);

        if (null != resources) {
            resources.remove(resourceId);
            redisHashMapAdapter.put(key, subKey, resources);
        }

        Message message = Message.build(userAuthIdMapper.removeAuthId(userId, authCode.index, resourceId));

        return message;
    }

    @Override
    public Message addResource(Long userId, AuthCodeEnum authCode, List<String> resources) {
        Preconditions.checkArgument(null != resources);

        KeyGenerator key = buildKey(userId);
        String subKey = String.valueOf(authCode.index);

        List<String> existsResources = redisHashMapAdapter.get(key, subKey, typeReference);

        if (null == existsResources) {
            existsResources = Lists.newArrayList();
        }
        existsResources.addAll(resources);

        redisHashMapAdapter.put(key, subKey, existsResources);

        List<UserAuthIdBean> userAuthIdBeans = Lists.newArrayList();
        Date now = new Date();
        for (String resource : resources) {
            UserAuthIdBean entity = new UserAuthIdBean();
            entity.setAuthCode(authCode.index);
            entity.setId(SequenceHolder.nextLongId());
            entity.setAuthId(resource);
            entity.setUserId(userId);
            entity.setCreateTime(now);

            userAuthIdBeans.add(entity);
        }

        int successTotal = 0;
        //批量操作
        for (UserAuthIdBean userAuthIdBean : userAuthIdBeans) {
            userAuthIdMapper.insert(userAuthIdBean);
            successTotal++;
        }

        boolean success = successTotal == userAuthIdBeans.size();

        return Message.build(success);
    }

    private KeyGenerator buildKey(Long userId) {
        return AuthRedisKey.USER_AUTH_RESOURCE.copy().appendKey(userId.toString());
    }
}

==> egg-user-service-1.0.0-20211019.060008-15-sources.jar: com/bxm/egg/user/auth/impl/UserAuthCodeServiceImpl.java <==
package com.bxm.egg.user.auth.impl;

import com.bxm.egg.common.enums.AuthCodeEnum;
import com.bxm.egg.common.enums.RoleCodeEnum;
import com.bxm.egg.user.auth.UserAuthCodeService;
import com.bxm.egg.user.constant.AuthRedisKey;
import com.bxm.egg.user.dto.auth.UserAuthResourceDTO;
import com.bxm.egg.user.integration.MessageFacadeIntegrationService;
import com.bxm.egg.user.mapper.auth.UserAuthCodeMapper;
import com.bxm.egg.user.mapper.auth.UserAuthIdMapper;
import com.bxm.egg.user.mapper.auth.UserRoleAuthMapper;
import com.bxm.egg.user.mapper.auth.UserRoleMapper;
import com.bxm.egg.user.model.vo.auth.UserAuthCodeBean;
import com.bxm.egg.user.model.vo.auth.UserAuthIdBean;
import com.bxm.egg.user.model.vo.auth.UserRoleAuthBean;
import com.bxm.egg.user.model.vo.auth.UserRoleBean;
import com.bxm.newidea.component.bo.Message;
import com.bxm.newidea.component.redis.KeyGenerator;
import com.bxm.newidea.component.redis.RedisHashMapAdapter;
import com.bxm.newidea.component.tools.BitOperatorUtil;
import com.bxm.newidea.component.tools.StringUtils;
import com.bxm.newidea.component.uuid.config.SequenceHolder;
import com.fasterxml.jackson.core.type.TypeReference;
import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
public class UserAuthCodeServiceImpl implements UserAuthCodeService {

    private RedisHashMapAdapter redisHashMapAdapter;

    private UserAuthCodeMapper userAuthCodeMapper;

    private UserAuthIdMapper userAuthIdMapper;

    private UserRoleAuthMapper userRoleAuthMapper;

    private UserRoleMapper userRoleMapper;

    private MessageFacadeIntegrationService messageSender;

    public UserAuthCodeServiceImpl(RedisHashMapAdapter redisHashMapAdapter,
                                   UserAuthCodeMapper userAuthCodeMapper,
                                   UserAuthIdMapper userAuthIdMapper,
                                   UserRoleAuthMapper userRoleAuthMapper,
                                   UserRoleMapper userRoleMapper,
                                   MessageFacadeIntegrationService messageSender) {
        this.redisHashMapAdapter = redisHashMapAdapter;
        this.userAuthCodeMapper = userAuthCodeMapper;
        this.userAuthIdMapper = userAuthIdMapper;
        this.userRoleAuthMapper = userRoleAuthMapper;
        this.userRoleMapper = userRoleMapper;
        this.messageSender = messageSender;
    }

    private TypeReference<List<String>> typeReference = new TypeReference<List<String>>() {
    };

    @Override
    public Long[] getAuthCombineCode(Long userId) {
        String combineCode = redisHashMapAdapter.get(AuthRedisKey.USER_AUTH_CODE, userId.toString(), String.class);

        Long[] authCodeArray;

        if (null == combineCode) {
            authCodeArray = new Long[]{0L};
        } else {
            String[] authCodeStrArray = combineCode.split(",");
            authCodeArray = new Long[authCodeStrArray.length];

            for (int i = 0; i < authCodeStrArray.length; i++) {
                authCodeArray[i] = Long.valueOf(authCodeStrArray[i]);
            }
        }

        return authCodeArray;
    }

    @Override
    public List<UserAuthResourceDTO> getAuthResources(Long userId) {
        Map<String, List<String>> authResourceMap = redisHashMapAdapter.entries(buildKey(userId), typeReference);

        if (null == authResourceMap) {
            authResourceMap = reloadResourceIdCache(userId);
        }

        return authResourceMap.entrySet().stream().map(entry ->
                new UserAuthResourceDTO(Integer.valueOf(entry.getKey()), entry.getValue()))
                .collect(Collectors.toList());
    }

    /**
     * 重新加载权限ID缓存数据
     *
     * @param userId 用户ID
     * @return 缓存数据
     */
    private Map<String, List<String>> reloadResourceIdCache(Long userId) {
        Map<String, List<String>> cacheMap = Maps.newHashMap();

        List<UserAuthIdBean> userAuthIdBeans = userAuthIdMapper.queryAuthIds(userId);

        userAuthIdBeans.forEach(item -> {
            String key = item.getAuthCode() + "";
            List<String> resourceIds = cacheMap.get(key);
            if (null == resourceIds) {
                resourceIds = Lists.newArrayList();
            }
            resourceIds.add(item.getAuthId());
            cacheMap.put(key, resourceIds);
        });

        KeyGenerator key = buildKey(userId);
        redisHashMapAdapter.remove(key);
        redisHashMapAdapter.putAll(key, cacheMap);

        return cacheMap;
    }

    @Override
    public Message addRole(Long userId, RoleCodeEnum role) {
        Preconditions.checkArgument(null != role);

        //设置用户拥有的角色
        UserRoleBean roleBean = new UserRoleBean();
        roleBean.setId(SequenceHolder.nextLongId());
        roleBean.setCreateTime(new Date());
        roleBean.setRoleCode(role.name());
        roleBean.setUserId(userId);

        userRoleMapper.insert(roleBean);

        //获取角色对应的操作权限列表
        List<UserRoleAuthBean> authBeans = userRoleAuthMapper.queryByCode(role.name());
        //设置用户拥有的权限集合
        Long[] authCombineCode = getAuthCombineCode(userId);
        for (UserRoleAuthBean userRoleAuth : authBeans) {
            AuthCodeEnum codeEnum = AuthCodeEnum.getByIndex(userRoleAuth.getAuthCode());
            authCombineCode = changeUserAuthCode(authCombineCode, codeEnum, true);
        }

        Message message = saveAuthCode(authCombineCode, userId);

        return message;
    }

    @Override
    public Message removeRole(Long userId, RoleCodeEnum role) {
        Preconditions.checkArgument(null != role);
        Preconditions.checkArgument(null != userId);

        //删除用户拥有的角色
        userRoleMapper.removeUserRole(userId, role.name());

        //获取角色对应的操作权限列表
        List<UserRoleAuthBean> authBeans = userRoleAuthMapper.queryByCode(role.name());
        //设置用户拥有的权限集合
        Long[] authCombineCode = getAuthCombineCode(userId);
        for (UserRoleAuthBean userRoleAuth : authBeans) {
            AuthCodeEnum codeEnum = AuthCodeEnum.getByIndex(userRoleAuth.getAuthCode());
            authCombineCode = changeUserAuthCode(authCombineCode, codeEnum, false);
        }

        //移除用户拥有的权限-资源信息
        //批量操作
        for (UserRoleAuthBean authBean : authBeans) {
            userAuthIdMapper.removeAuth(userId, authBean.getAuthCode());
        }

        reloadResourceIdCache(userId);

        return saveAuthCode(authCombineCode, userId);
    }

    @Override
    public Message addAuthCode(Long userId, AuthCodeEnum authCode) {
        Long[] authCombineCode = getAuthCombineCode(userId);
        authCombineCode = changeUserAuthCode(authCombineCode, authCode, true);

        return saveAuthCode(authCombineCode, userId);
    }

    private Message saveAuthCode(Long[] authCodeArray, Long userId) {
        String authCombineCode = StringUtils.join(authCodeArray, ",");
        redisHashMapAdapter.put(AuthRedisKey.USER_AUTH_CODE, userId.toString(), authCombineCode);

        int rows;
        UserAuthCodeBean entity = userAuthCodeMapper.queryByUserId(userId);
        if (entity == null) {
            entity = new UserAuthCodeBean();
            entity.setId(SequenceHolder.nextLongId());
            entity.setCreateTime(new Date());
            entity.setAuthCombineCode(authCombineCode);
            entity.setUserId(userId);

            rows = userAuthCodeMapper.insert(entity);
        } else {
            entity.setAuthCombineCode(authCombineCode);
            entity.setModifyTime(new Date());

            rows = userAuthCodeMapper.updateByPrimaryKey(entity);
        }

        return Message.build(rows);
    }

    /**
     * 变更用户持有的权限合集
     *
     * @param authCombineCode 权限编码合集
     * @param authCode        变更的授权编码
     * @param isAdd           true表示添加，false表示移除
     * @return 变更后的权限合集
     */
    private Long[] changeUserAuthCode(Long[] authCombineCode, AuthCodeEnum authCode, boolean isAdd) {
        return BitOperatorUtil.setBitToArray(authCombineCode, authCode.index, isAdd);
    }

    @Override
    public Message removeAuthCode(Long userId, AuthCodeEnum authCode) {
        Long[] authCombineCode = getAuthCombineCode(userId);
        authCombineCode = changeUserAuthCode(authCombineCode, authCode, false);

        userAuthIdMapper.removeAuth(userId, authCode.index);
        redisHashMapAdapter.remove(buildKey(userId), authCode.index + "");

        return saveAuthCode(authCombineCode, userId);
    }

    @Override
    public Message addResource(Long userId, AuthCodeEnum authCode, String resourceId) {
        return addResource(userId, authCode, ImmutableList.of(resourceId));
    }

    @Override
    public Boolean hasAuth(Long userId, AuthCodeEnum codeEnum) {
        int authCode = codeEnum.index;
        Long[] parseAuthCodeArray = getAuthCombineCode(userId);

        return BitOperatorUtil.hasBit(parseAuthCodeArray, authCode);
    }

    @Override
    public Message removeResource(Long userId, AuthCodeEnum authCode, String resourceId) {
        KeyGenerator key = buildKey(userId);
        String subKey = String.valueOf(authCode.index);

        List<String> resources = redisHashMapAdapter.get(key, subKey, typeReference);

        if (null != resources) {
            resources.remove(resourceId);
            redisHashMapAdapter.put(key, subKey, resources);
        }

        Message message = Message.build(userAuthIdMapper.removeAuthId(userId, authCode.index, resourceId));

        return message;
    }

    @Override
    public Message addResource(Long userId, AuthCodeEnum authCode, List<String> resources) {
        Preconditions.checkArgument(null != resources);

        KeyGenerator key = buildKey(userId);
        String subKey = String.valueOf(authCode.index);

        List<String> existsResources = redisHashMapAdapter.get(key, subKey, typeReference);

        if (null == existsResources) {
            existsResources = Lists.newArrayList();
        }
        existsResources.addAll(resources);

        redisHashMapAdapter.put(key, subKey, existsResources);

        List<UserAuthIdBean> userAuthIdBeans = Lists.newArrayList();
        Date now = new Date();
        for (String resource : resources) {
            UserAuthIdBean entity = new UserAuthIdBean();
            entity.setAuthCode(authCode.index);
            entity.setId(SequenceHolder.nextLongId());
            entity.setAuthId(resource);
            entity.setUserId(userId);
            entity.setCreateTime(now);

            userAuthIdBeans.add(entity);
        }

        int successTotal = 0;
        //批量操作
        for (UserAuthIdBean userAuthIdBean : userAuthIdBeans) {
            userAuthIdMapper.insert(userAuthIdBean);
            successTotal++;
        }

        boolean success = successTotal == userAuthIdBeans.size();

        return Message.build(success);
    }

    private KeyGenerator buildKey(Long userId) {
        return AuthRedisKey.USER_AUTH_RESOURCE.copy().appendKey(userId.toString());
    }
}
